// Description: generates events with the size of the simulator window
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//</ul>
//
// Category: System
// Author: Alex Hill
// Revision: 05/10/02
//
#include <pfcave.h>
#include <Performer/pf.h>
#include <Performer/pf/pfChannel.h>
#include <ygNetMessages.h>
#include "CAVEWindow.h"

using namespace std;

extern "C" ygNode* construct_CAVEWindow(const char* name,bool master) { return new CAVEWindow(name,master); }
char * CAVEWindow_dependencies = "YG DSO Dependencies: ygNode";

CAVEWindow::CAVEWindow(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("CAVEWindow");
    xDim = 0;
    yDim = 0;
    lastXDim = 0;
    lastYDim = 0;
    addNetMessage("xdim",&xDim,YG_NET_INT);
    addNetMessage("ydim",&yDim,YG_NET_INT);
    addEvent("changed");
    }

CAVEWindow::~CAVEWindow(void)
    {
    }


void CAVEWindow::reset(void)
    {
    ygNode::reset();
    }


void CAVEWindow::app(void)
    {
    if (CAVEConfig->Simulator)
        {
        pfCAVEMasterChan()->getPWin()->getSize(&xDim,&yDim);
        if (xDim != lastXDim || yDim != lastYDim)
            {
            // the dimensions of the simulator window have changed
            eventOccurred("changed");
            }
        lastXDim = xDim;
        lastYDim = yDim;
        }
    ygNode::app();
    }
