// Description: a transform that orients itself about an axis
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> the model should be oriented with the negative Y axis
//<li> the point mode rotates the model about a point
//<li> the optional axis is the model up vector in point mode
//<li> the axis mode rotates the model about an axis
//</ul>
//
// Category: Transformation
// Author: Alex Hill
// Revision: 11/01/01
//
#include <ygNetMessages.h>
#include "orientAxis.h"

#define YG_AXIS_X 901
#define YG_AXIS_Y 902
#define YG_AXIS_Z 903
#define YG_AXIS_XY 904
#define YG_AXIS_YZ 905
#define YG_AXIS_XZ 906
#define YG_AXIS_XYZ 907

extern "C" ygNode* construct_orientAxis(const char* name,bool master) { return new orientAxis(name,master); }

orientAxis::orientAxis(const char* name,bool master) : ygTransform(name,master)
    {
    setClassName("orientAxis");
    axis = 0;
    addNetMessage("axis",&axis,YG_NET_INT);
    addNetMessageMap("axis","off",0);
    addNetMessageMap("axis","x",YG_AXIS_X);
    addNetMessageMap("axis","y",YG_AXIS_Y);
    addNetMessageMap("axis","z",YG_AXIS_Z);
    addNetMessageMap("axis","xy",YG_AXIS_XY);
    addNetMessageMap("axis","yz",YG_AXIS_YZ);
    addNetMessageMap("axis","xz",YG_AXIS_XZ);
    addNetMessageMap("axis","xyz",YG_AXIS_XYZ);
    }

void orientAxis::reset(void)
    {
    //set axis to the YZ axis
    resetNetMessage("axis");
    ygTransform::reset();
    }


void orientAxis::message(const ygMessage& msg)
    {
    //clear or set the node and optional user body part
    if (msg == "axis")
        {
        return;
        }
    else
        ygTransform::message(msg);
    }


void orientAxis::app(void)
    {
    pfMatrix matrix;
    matrix.makeIdent();
    if (axis)
        {
        pfMatrix parent;
        getParentTransform(parent);
        pfVec3d row;
        pfVec3 size;
        row.set(parent.mat[0][0],parent.mat[0][1],parent.mat[0][2]);
        size[0] = row.length();
        row.set(parent.mat[1][0],parent.mat[1][1],parent.mat[1][2]);
        size[1] = row.length();
        row.set(parent.mat[2][0],parent.mat[2][1],parent.mat[2][2]);
        size[2] = row.length();
        pfMatrix unscaled;
        unscaled.preScale(1.0/size[0],1.0/size[1],1.0/size[2],parent);
        pfCoord coord;
        unscaled.getOrthoCoord(&(coord));
        if (coord.hpr[0] == -180.0 && coord.hpr[2] == 180.0)
            {
            coord.hpr[0] = 0.0;
            if (coord.hpr[1] >= 0.0)
                coord.hpr[1] = 180.0 - coord.hpr[1];
            else
                coord.hpr[1] = -(180.0 + coord.hpr[1]);
            coord.hpr[2] = 0.0;
            }
        if (axis == YG_AXIS_X) 
            matrix.makeEuler(coord.hpr[0],0,coord.hpr[2]);
        else if (axis == YG_AXIS_Y)
            matrix.makeEuler(coord.hpr[0],coord.hpr[1],0);
        else if (axis == YG_AXIS_Z)
            matrix.makeEuler(0,coord.hpr[1],coord.hpr[2]);
        else if (axis == YG_AXIS_XY)
            matrix.makeEuler(coord.hpr[0],0,0);
        else if (axis == YG_AXIS_YZ)
            matrix.makeEuler(0,coord.hpr[1],0);
        else if (axis == YG_AXIS_XZ)
            matrix.makeEuler(0,0,coord.hpr[2]);
        else if (axis == YG_AXIS_XYZ)
            matrix.makeIdent();
        pfMatrix inverse;
        parent.makeEuler(coord.hpr[0],coord.hpr[1],coord.hpr[2]);
        inverse.invertAff(parent);
        matrix.postMult(inverse);
        }
    //set the dcs matrix
    setMatrix(matrix);
    ygTransform::app();
    }
