// Description: a node that creates a 3D string of text
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> the initial text size is 0.5 oriented in the XZ plane
//<li> current available fonts are:
//<ul>
//<li> Courier-Bold
//<li> Courier-BoldOblique
//<li> Courier-Oblique
//<li> Helvetica
//<li> Helvetica-Bold
//<li> Helvetica-BoldOblique
//<li> Helvetica-Narrow
//<li> Helvetica-Narrow-BoldOblique
//<li> Helvetica-Narrow-Oblique
//<li> Helvetica-Oblique
//<li> Mistr
//<li> Stoke-Bold
//<li> Stoke-Roman
//<li> Times-Bold
//<li> Times-BoldItalic
//<li> Times-Eflin
//<li> Times-Roman
//</ul>
//</ul>
//
// Category: Geometry
// Author: Alex Hill
// Revision: 04/02/03
//
#include <Performer/pfdu.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfString.h>
#include <Performer/pf/pfText.h>
#include <Performer/pf/pfGroup.h>
#include <Performer/pr/pfMaterial.h>
#include <ygNetMessages.h>
#include <ygWorld.h>
#include "text.h"

using namespace std;

extern "C" ygNode* construct_text(const char* name,bool master) { return new text(name,master); }


text::text(const char* name,bool master) : ygGeometry(name,master)
    {
    setClassName("text",true);
    pfFont *font = pfdLoadFont_type1("/usr/share/Performer/data/Times-Elfin",PFDFONT_EXTRUDED);
    string = new pfString;
    /*pfMaterial* mat = new pfMaterial;
    mat->setSide(PFMTL_BOTH);
    mat->setColorMode(PFMTL_BOTH,PFMTL_CMODE_OFF);
    mat->setColor(PFMTL_EMISSION,1,0,0);
    gstate = new pfGeoState;
    gstate->setAttr(PFSTATE_FRONTMTL,mat);
    gstate->setAttr(PFSTATE_BACKMTL,mat);
    string->setGState(gstate);*/
    string->setFont(font);
    text_ = new pfText;
    //set draw mask
    text_->setTravMask(PFTRAV_DRAW,0xffffffff,PFTRAV_SELF|PFTRAV_DESCEND,PF_SET);
    //set intersection traversal mask
    text_->setTravMask(PFTRAV_ISECT, YG_ISECT_ALLMASK, PFTRAV_SELF|PFTRAV_DESCEND|PFTRAV_IS_CACHE, PF_SET);
    text_->addString(string);
    text_->setName(name);
    object->addChild(text_);
    netFont = "Times-Elfin";
    netColor.set(0,0,0,1);
    width = 0;
    lastWidth = 0;
    string->setColor(netColor[0],netColor[1],netColor[2],netColor[3]);
    netStyle = PFDFONT_FILLED;
    netJustify = PFSTR_LEFT;
    addNetMessage("string",&netString,YG_NET_STRING);
    addNetMessage("color",&netColor,YG_NET_VEC4);
    addNetMessage("font",&netFont,YG_NET_STRING);
    addNetMessage("font",&netStyle,YG_NET_INT);
    addNetMessageMap("font","filled",PFDFONT_FILLED);
    addNetMessageMap("font","extruded",PFDFONT_EXTRUDED);
    addNetMessageMap("font","textured",PFDFONT_TEXTURED);
    addNetMessageMap("font","outlined",PFDFONT_OUTLINED);
    addNetMessage("justify",&netJustify,YG_NET_INT);
    addNetMessageMap("justify","left",PFSTR_LEFT);
    addNetMessageMap("justify","center",PFSTR_CENTER);
    addNetMessageMap("justify","right",PFSTR_RIGHT);
    addNetMessage("width",&width,YG_NET_FLOAT);
    dontSendNetMessage("width");
    dontSaveNetMessage("width");
    addEvent("changed");
    }

text::~text(void)
    {
    }


void text::reset(void)
    {
    resetNetMessage("string");
    string->setString(netString.c_str());
    text_->setBound(NULL,PFBOUND_DYNAMIC);
    text_->setName(name().c_str()); 
    //set default font to "Times-Elfin"
    pfFont *font = pfdLoadFont_type1("/usr/share/Performer/data/Times-Elfin",PFDFONT_EXTRUDED);
    //set default style to extruded
    resetNetMessage("font");
    string->setFont(font);
    //set color to (0,0,0,1)
    resetNetMessage("color");;
    string->setColor(netColor[0],netColor[1],netColor[2],netColor[3]);
    width = 0;
    lastWidth = 0;
    ygGeometry::reset();
    }


void text::message(const ygMessage& msg)
    {
    //set text string to be generated
    if (msg == "string")
        {
        string->setString(netString.c_str());
        /*for (int i=0; i<string->getStringLength(); i++)
            {
            pfGeoSet* chr = (pfGeoSet*)string->getCharGSet(i);
            if (chr)
                chr->setGState(gstate);
            }*/
        text_->setBound(NULL,PFBOUND_DYNAMIC);
        pfSphere sphere;
        text_->getBound(&sphere);
        if (sphere.radius > 0)
            width = 2*(sphere.radius);
        else
            width = 0;
        }
    //set the color of the text
    else if (msg == "color")
        {
        string->setColor(netColor[0],netColor[1],netColor[2],netColor[3]);
        }
    //set the font and optional style
    else if (msg == "font")
        {
        if (netFont.length() > 0)
            {
            ygString fontPath("/usr/share/Performer/data/");
            fontPath += netFont;            
            pfFont *font = pfdLoadFont_type1(fontPath.c_str(),netStyle);
            string->setFont(font);
            }
        }
    //set the font justification
    else if (msg == "justify")
        {
        string->setMode(PFSTR_JUSTIFY,netJustify);
        }
    else
        ygGeometry::message(msg);
    }


void text::app(void)
    {
    if (width != lastWidth)
        {
        ygString args;
        args = "width=";
        char val[32];
        sprintf(val,"%f",width);
        args += val;
        //the width of the text has changed
        eventOccurred("changed",args);
        lastWidth = width;
        }
    ygGeometry::app();
    }
